@@ -54,6 +54,8 @@ public class DelegatingBeanAspectFactory implements AspectFactory, KernelControl
    
    protected KernelControllerContext context;
    
+   protected AspectFactory aspectFactory;
+   
    public DelegatingBeanAspectFactory(String name, BeanFactory factory, Element element)
    {
       this.name = name;
@@ -112,44 +114,47 @@ public class DelegatingBeanAspectFactory implements AspectFactory, KernelControl
       return factory.createPerJoinpoint(advisor, instanceAdvisor, jp);
    }
 
-   protected AspectFactory doCreate()
+   protected synchronized AspectFactory doCreate()
    {
       try
       {
-         log.debug("Creating advice " + name);
-
-         PushedClassLoaderMetaData pcmd = null;
-         if (((GenericBeanFactory)factory).getClassLoader() == null)
+         if (aspectFactory == null)
          {
-            pcmd = new PushedClassLoaderMetaData();
-            ((GenericBeanFactory)factory).setClassLoader(pcmd);
-         }
-         
-         Object object = null;
-         try
-         {
-            //Try without looking at the context first which is what shold be used when running scoped in AS
-            object = factory.createBean();
-         }
-         catch(Throwable t)
-         {
-            if (pcmd != null)
+            log.debug("Creating advice " + name);
+   
+            PushedClassLoaderMetaData pcmd = null;
+            if (((GenericBeanFactory)factory).getClassLoader() == null)
             {
-               pcmd.setLookAtContext(true);
+               pcmd = new PushedClassLoaderMetaData();
+               ((GenericBeanFactory)factory).setClassLoader(pcmd);
             }
-            else
+            
+            Object object = null;
+            try
             {
-               throw new RuntimeException(t);
+               //Try without looking at the context first which is what shold be used when running scoped in AS
+               object = factory.createBean();
+            }
+            catch(Throwable t)
+            {
+               if (pcmd != null)
+               {
+                  pcmd.setLookAtContext(true);
+               }
+               else
+               {
+                  throw new RuntimeException(t);
+               }
+               object = factory.createBean();
+            }
+   
+            aspectFactory = (AspectFactory)object;
+            if (aspectFactory instanceof XmlLoadable)
+            {
+               ((XmlLoadable)aspectFactory).importXml(element);
             }
-            object = factory.createBean();
          }
-
-         AspectFactory fac = (AspectFactory)object;
-         if (fac instanceof XmlLoadable)
-         {
-            ((XmlLoadable)fac).importXml(element);
-         }
-         return fac;
+         return aspectFactory;
       }
       catch (Throwable throwable)
       {
@@ -177,37 +182,30 @@ public class DelegatingBeanAspectFactory implements AspectFactory, KernelControl
       public ValueMetaData getClassLoader()
       {
          ClassLoader loader = null; 
-         if (loader == null)
+         if (lookAtContext && context != null)
          {
-            if (lookAtContext && context != null)
+            try
+            {
+               loader = context.getClassLoader();
+            }
+            catch (Throwable t)
+            {
+               log.trace("Unable to retrieve classloader from " + context);
+            }
+            
+            if (loader == null)
             {
                try
                {
-                  loader = context.getClassLoader();
+                  loader = Configurator.getClassLoader(((GenericBeanFactory)factory).getClassLoader());
                }
-               catch (Throwable t)
+               catch (Throwable e)
                {
-                  log.trace("Unable to retrieve classloader from " + context);
-               }
-               
-               if (loader == null)
-               {
-                  try
-                  {
-                     loader = Configurator.getClassLoader(((GenericBeanFactory)factory).getClassLoader());
-                  }
-                  catch (Throwable e)
-                  {
-                     log.trace("Unable to retrieve classloader from " + factory);
-                  }
+                  log.trace("Unable to retrieve classloader from " + factory);
                }
             }
-            return loader != null ?  new AbstractValueMetaData(loader) : null;
-         }
-         else
-         {
-            return new AbstractValueMetaData(loader);
          }
+         return loader != null ?  new AbstractValueMetaData(loader) : null;
       }
       
    }
