@@ -21,6 +21,8 @@
 */ 
 package org.jboss.aop.microcontainer.beans.metadata;
 
+import javax.xml.bind.annotation.XmlType;
+
 import org.jboss.aop.advice.AdviceType;
 
 /**
@@ -28,8 +30,11 @@ import org.jboss.aop.advice.AdviceType;
  * @author <a href="kabir.khan@jboss.com">Kabir Khan</a>
  * @version $Revision: 1.1 $
  */
+@XmlType(name="afterAdviceType", propOrder={})
 public class AfterAdviceData extends AdviceData
 {
+   private static final long serialVersionUID = 1L;
+
    public AfterAdviceData()
    {
       setType(AdviceType.AFTER);
