@@ -59,7 +59,7 @@ import org.jboss.util.collection.CollectionsFactory;
  *
  * @author <a href="adrian@jboss.com">Adrian Brock</a>
  * @author <a href="ales.justin@jboss.com">Ales Justin</a>
- * @version $Revision: 78339 $
+ * @version $Revision: 96589 $
  */
 public class AbstractController extends JBossObject implements Controller, ControllerStateModel, GraphController, AbstractControllerMBean
 {
@@ -84,7 +84,7 @@ public class AbstractController extends JBossObject implements Controller, Contr
    /** The error contexts Map<Name, ControllerContext> */
    private Map<Object, ControllerContext> errorContexts = new ConcurrentHashMap<Object, ControllerContext>();
 
-   /** The contexts that are currently being installed */
+   /** The contexts that are currently being resolved/installed */
    private Set<ControllerContext> installing = new CopyOnWriteArraySet<ControllerContext>();
 
    /** The parent controller */
@@ -104,7 +104,7 @@ public class AbstractController extends JBossObject implements Controller, Contr
    private boolean collectStats = false;
    
    /** The install stats */
-   private StateStatistics installStats = null;
+   private volatile StateStatistics installStats = null;
    
    /**
     * Create an abstract controller
@@ -252,9 +252,12 @@ public class AbstractController extends JBossObject implements Controller, Contr
 
    public String listStateTimes(boolean details)
    {
-      if (installStats == null)
-         return "No statistics available";
-      return installStats.listTimes(details);
+      synchronized (this)
+      {
+         if (installStats == null)
+            return "No statistics available";
+         return installStats.listTimes(details);
+      }
    }
 
    public void addState(ControllerState state, ControllerState before)
@@ -586,7 +589,7 @@ public class AbstractController extends JBossObject implements Controller, Contr
    {
       Map<ControllerState, ControllerContextAction> map = createAliasActions();
       ControllerContextActions actions = new AbstractControllerContextActions(map);
-      AliasControllerContext context = new InnerAliasControllerContext(alias, original, actions);
+      AliasControllerContext context = new InnerAliasControllerContext(alias, getId(), original, actions);
       preAliasInstall(context);
       install(context);
       // is alias in error
@@ -726,8 +729,9 @@ public class AbstractController extends JBossObject implements Controller, Contr
             }
          }
 
-         if (ControllerMode.AUTOMATIC.equals(context.getMode()))
-            context.setRequiredState(ControllerState.INSTALLED);
+         // set the required state
+         ControllerMode mode = context.getMode();
+         context.setRequiredState(mode.getRequiredState());
 
          if (trace)
             log.trace("Installing " + context.toShortString());
@@ -1036,9 +1040,10 @@ public class AbstractController extends JBossObject implements Controller, Contr
    {
       boolean resolutions = false;
       Set<ControllerContext> unresolved = contextsByState.get(fromState);
-      Set<ControllerContext> resolved = resolveContexts(unresolved, toState, trace);
+      Set<ControllerContext> resolved = resolveContexts(unresolved, fromState, toState, trace);
       if (resolved.isEmpty() == false)
       {
+         Set<ControllerContext> toProcess = new HashSet<ControllerContext>();
          for (ControllerContext context : resolved)
          {
             Object name = context.getName();
@@ -1046,34 +1051,62 @@ public class AbstractController extends JBossObject implements Controller, Contr
             {
                if (trace)
                   log.trace("Skipping already installed " + name + " for " + toState.getStateString());
-            }
-            else if (installing.add(context) == false)
-            {
-               if (trace)
-                  log.trace("Already installing " + name + " for " + toState.getStateString());
+               installing.remove(context);
             }
             else
             {
-               try
+               toProcess.add(context);
+            }
+         }
+         try
+         {
+            if (toProcess.isEmpty() == false)
+            {
+               for (Iterator<ControllerContext> iter = toProcess.iterator(); iter.hasNext(); )
                {
-                  if (trace)
-                     log.trace("Dependencies resolved " + name + " for " + toState.getStateString());
-
-                  if (incrementState(context, trace))
+                  ControllerContext context = iter.next();
+                  iter.remove();
+                  Object name = context.getName();
+                  try
                   {
-                     resolutions = true;
-                     if (trace)
-                        log.trace(name + " " + toState.getStateString());
+                     if (fromState.equals(context.getState()) == false)
+                     {
+                        if (trace)
+                           log.trace("Skipping already installed " + name + " for " + toState.getStateString());
+                     }
+                     else
+                     {
+                        if (trace)
+                           log.trace("Dependencies resolved " + name + " for " + toState.getStateString());
+
+                        if (incrementState(context, trace))
+                        {
+                           resolutions = true;
+                           if (trace)
+                              log.trace(name + " " + toState.getStateString());
+                        }
+                        
+                     }
+                  }
+                  finally
+                  {
+                     installing.remove(context);
                   }
-               }
-               finally
-               {
-                  installing.remove(context);
                }
             }
          }
+         finally
+         {
+            // If we get here something has gone seriously wrong,
+            // but try to tidyup as much state as possible
+            if (toProcess.isEmpty() == false)
+            {
+               for (ControllerContext context : toProcess)
+                  installing.remove(context);
+            }
+         }
       }
-
+      
       return resolutions;
    }
 
@@ -1083,11 +1116,12 @@ public class AbstractController extends JBossObject implements Controller, Contr
     * This method must be invoked with the write lock taken
     *
     * @param contexts the contexts
-    * @param state    the state
+    * @param fromState the from state
+    * @param toState   the to state
     * @param trace    whether trace is enabled
     * @return the set of resolved contexts
     */
-   protected Set<ControllerContext> resolveContexts(Set<ControllerContext> contexts, ControllerState state, boolean trace)
+   protected Set<ControllerContext> resolveContexts(Set<ControllerContext> contexts, ControllerState fromState, ControllerState toState, boolean trace)
    {
       HashSet<ControllerContext> result = new HashSet<ControllerContext>();
 
@@ -1095,22 +1129,40 @@ public class AbstractController extends JBossObject implements Controller, Contr
       {
          for (ControllerContext ctx : contexts)
          {
-            if (advance(ctx))
+            Object name = ctx.getName();
+            if (fromState.equals(ctx.getState()) == false)
+            {
+               if (trace)
+                  log.trace("Skipping already installed " + name + " for " + toState.getStateString());
+            }
+            else if (installing.add(ctx) == false)
+            {
+               if (trace)
+                  log.trace("Already installing " + name + " for " + toState.getStateString());
+            }
+            else if (advance(ctx))
             {
                DependencyInfo dependencies = ctx.getDependencyInfo();
                try
                {
-                  if (dependencies == null || dependencies.resolveDependencies(this, state))
+                  if (dependencies == null || dependencies.resolveDependencies(this, toState))
                      result.add(ctx);
+                  else
+                     installing.remove(ctx);
                }
                catch (Throwable error)
                {
-                  log.error("Error resolving dependencies for " + state.getStateString() + ": " + ctx.toShortString(), error);
+                  installing.remove(ctx);
+                  log.error("Error resolving dependencies for " + toState.getStateString() + ": " + ctx.toShortString(), error);
                   uninstallContext(ctx, ControllerState.NOT_INSTALLED, trace);
                   errorContexts.put(ctx.getName(), ctx);
                   ctx.setError(error);
                }
             }
+            else
+            {
+               installing.remove(ctx);
+            }
          }
       }
 
@@ -1197,7 +1249,8 @@ public class AbstractController extends JBossObject implements Controller, Contr
 
                               for (ControllerContext dependent : dependents)
                               {
-                                 if (isBeforeState(dependent.getState(), whenRequired) == false)
+                                 boolean selfDependency = (dependent == context);
+                                 if (selfDependency == false && isBeforeState(dependent.getState(), whenRequired) == false)
                                     uninstallContext(dependent, whenRequired, trace);
                               }
                            }
@@ -1372,7 +1425,7 @@ public class AbstractController extends JBossObject implements Controller, Contr
       {
          Map<Object, Set<CallbackItem<?>>> map = (isInstallPhase ? installCallbacks : uninstallCallbacks);
          Set<CallbackItem<?>> callbacks = map.get(name);
-         return callbacks != null ? callbacks : new HashSet<CallbackItem<?>>();
+         return callbacks != null ? callbacks : Collections.<CallbackItem<?>>emptySet();
       }
       finally
       {
@@ -1445,7 +1498,8 @@ public class AbstractController extends JBossObject implements Controller, Contr
          if (dependencyInfo != null && dependencyInfo.isAutowireCandidate())
          {
             // match callbacks by name
-            Set<CallbackItem<?>> existingCallbacks = getCallbacks(context.getName(), isInstallPhase);
+            Set<CallbackItem<?>> existingCallbacks = new HashSet<CallbackItem<?>>(); 
+            existingCallbacks.addAll(getCallbacks(context.getName(), isInstallPhase));
             // match by classes
             Collection<Class<?>> classes = getClassesImplemented(context.getTarget());
             if (classes != null && classes.isEmpty() == false)
@@ -1851,11 +1905,11 @@ public class AbstractController extends JBossObject implements Controller, Contr
 
    // --- alias dependency
 
-   private class InnerAliasControllerContext extends AbstractAliasControllerContext
+   private static class InnerAliasControllerContext extends AbstractAliasControllerContext
    {
-      private InnerAliasControllerContext(Object alias, Object original, ControllerContextActions actions)
+      private InnerAliasControllerContext(Object alias, String id, Object original, ControllerContextActions actions)
       {
-         super(alias, getId(), original, actions);
+         super(alias, id, original, actions);
       }
    }
 
