@@ -52,10 +52,14 @@ import org.jboss.reflect.spi.TypeInfoFactory;
  *
  * @author <a href="adrian@jboss.com">Adrian Brock</a>
  * @author <a href="ales.justin@jboss.com">Ales Justin</a>
- * @version $Revision: 81535 $
+ * @version $Revision: 96564 $
  */
 public class Configurator extends Config
 {
+   final static String[] NO_PARAMS = new String[0];
+   final static Object[] NO_PARAM_VALUES = new Object[0]; 
+   
+   
    /**
     * Instantiate and configure a bean
     *
@@ -573,7 +577,7 @@ public class Configurator extends Config
    public static String[] getParameterTypes(boolean trace, List<ParameterMetaData> parameters) throws Throwable
    {
       if (parameters == null)
-         return null;
+         return NO_PARAMS;
 
       String[] paramTypes = new String[parameters.size()];
       int x = 0;
@@ -596,7 +600,7 @@ public class Configurator extends Config
    public static String[] getParameterTypes(boolean trace, TypeInfo[] parameters) throws Throwable
    {
       if (parameters == null)
-         return null;
+         return NO_PARAMS;
 
       String[] paramTypes = new String[parameters.length];
       int x = 0;
@@ -618,7 +622,7 @@ public class Configurator extends Config
    public static Object[] getParameters(boolean trace, ClassLoader cl, TypeInfo[] pinfos, List<ParameterMetaData> parameters) throws Throwable
    {
       if (parameters == null)
-         return null;
+         return NO_PARAM_VALUES;
 
       Object[] params = new Object[parameters.size()];
       int x = 0;
@@ -692,7 +696,6 @@ public class Configurator extends Config
     * @return true when they can be assigned
     * @throws Throwable for any error
     */
-   @SuppressWarnings("unchecked")
    public static boolean isAssignable(TypeInfoFactory tif, ClassLoader cl, String[] typeNames, TypeInfo[] typeInfos) throws Throwable
    {
       if (cl == null)
