@@ -52,7 +52,7 @@ import org.jboss.kernel.spi.registry.KernelRegistryPlugin;
  * 
  * @author <a href="adrian@jboss.com">Adrian Brock</a>
  * @author <a href="ales.justin@jboss.com">Ales Justin</a>
- * @version $Revision: 83479 $
+ * @version $Revision: 104925 $
  */
 @SuppressWarnings("deprecation")
 public class AbstractKernelController extends ScopedController implements KernelController, KernelRegistryPlugin
@@ -107,6 +107,7 @@ public class AbstractKernelController extends ScopedController implements Kernel
       return context;
    }
 
+   @SuppressWarnings("unchecked")
    public KernelRegistryEntry getEntry(Object name)
    {
       List<KernelControllerContext> list;
@@ -400,6 +401,8 @@ public class AbstractKernelController extends ScopedController implements Kernel
                log.trace("Removing contex " + context + " to class: " + clazz);
             }
             classContext.contexts.remove(context);
+            if (classContext.contexts.isEmpty())
+               contextsByClass.remove(clazz);
          }
       }
       // traverse superclass
@@ -412,7 +415,7 @@ public class AbstractKernelController extends ScopedController implements Kernel
       }
    }
 
-   private class ClassContext
+   private static class ClassContext
    {
       private boolean used;
       private Set<KernelControllerContext> contexts;
